import 'package:permission_handler/permission_handler.dart' as p;

import '../components/show_dialogs.dart';
import '../waff_localizations.dart';

/// Request permission
///
/// ```
/// if (await RequestPermission.single(Permission.location,
//     'To use "People nearby" feature, must turn on location permissions.')) {
//   showAlertDialog('Success!');
// } else {
//   showAlertDialog('Error!');
// }
/// ```
abstract class RequestPermission {
  /// Single
  static Future<bool> single(p.Permission permission, String alert,
      {bool goAppSettings = false}) async {
    var status = await permission.request();
    if (status.isGranted) {
      return true;
    }

    if (goAppSettings && await _goAppSettings(permission, status, alert)) {
      return await single(permission, alert);
    }

    return false;
  }

  /// batch
  // static Future<Map<Permission, bool>> batch(
  //     Map<Permission, String> permissions) async {
  //   Map<Permission, bool> results = {};
  //
  //   for (var entry in permissions.entries) {
  //     PermissionStatus status = await entry.key.request();
  //     if (status.isGranted) {
  //       results.putIfAbsent(entry.key, () => true);
  //     } else {
  //       await _confirm(entry.key, status, entry.value);
  //       results.putIfAbsent(entry.key, () => false);
  //     }
  //   }
  //
  //   return results;
  // }

  /// open system settings
  static Future<bool> _goAppSettings(
      p.Permission permission, p.PermissionStatus status, String alert) async {
    if (!status.isPermanentlyDenied) {
      return false;
    }

    if (await showConfirmDialog(
        '$alert\n\n${WaffLocalizations.S('authorize_in_system_setting')}',
        yes: WaffLocalizations.S('system_setting'))) {
      if (await p.openAppSettings()) {
        // TODO: listening app cuts back to the foreground
        return true;
      }
    }

    return false;
  }
}

class Permission {
  /// Android: Calendar
  /// iOS: Calendar (Events)
  static const calendar = p.Permission.calendar;

  /// Android: Camera
  /// iOS: Photos (Camera Roll and Camera)
  static const camera = p.Permission.camera;

  /// Android: Contacts
  /// iOS: AddressBook
  static const contacts = p.Permission.contacts;

  /// Android: Fine and Coarse Location
  /// iOS: CoreLocation (Always and WhenInUse)
  static const location = p.Permission.location;

  /// Android:
  ///   When running on Android < Q: Fine and Coarse Location
  ///   When running on Android Q and above: Background Location Permission
  /// iOS: CoreLocation - Always
  static const locationAlways = p.Permission.locationAlways;

  /// Android: Fine and Coarse Location
  /// iOS: CoreLocation - WhenInUse
  static const locationWhenInUse = p.Permission.locationWhenInUse;

  /// Android: None
  /// iOS: MPMediaLibrary
  static const mediaLibrary = p.Permission.mediaLibrary;

  /// Android: Microphone
  /// iOS: Microphone
  static const microphone = p.Permission.microphone;

  /// Android: Phone
  /// iOS: Nothing
  static const phone = p.Permission.phone;

  /// Android: Nothing
  /// iOS: Photos
  /// iOS 14+ read & write access level
  static const photos = p.Permission.photos;

  /// Android: Nothing
  /// iOS: Photos
  /// iOS 14+ read & write access level
  static const photosAddOnly = p.Permission.photosAddOnly;

  /// Android: Nothing
  /// iOS: Reminders
  static const reminders = p.Permission.reminders;

  /// Android: Body Sensors
  /// iOS: CoreMotion
  static const sensors = p.Permission.sensors;

  /// Android: Sms
  /// iOS: Nothing
  static const sms = p.Permission.sms;

  /// Android: Microphone
  /// iOS: Speech
  static const speech = p.Permission.speech;

  /// Android: External Storage
  /// iOS: Access to folders like `Documents` or `Downloads`. Implicitly
  /// granted.
  static const storage = p.Permission.storage;

  /// Android: Ignore Battery Optimizations
  static const ignoreBatteryOptimizations =
      p.Permission.ignoreBatteryOptimizations;

  /// Android: Notification
  /// iOS: Notification
  static const notification = p.Permission.notification;

  /// Android: Allows an application to access any geographic locations
  /// persisted in the user's shared collection.
  static const accessMediaLocation = p.Permission.accessMediaLocation;

  /// When running on Android Q and above: Activity Recognition
  /// When running on Android < Q: Nothing
  /// iOS: Nothing
  static const activityRecognition = p.Permission.activityRecognition;

  /// The unknown only used for return type, never requested
  static const unknown = p.Permission.unknown;

  /// iOS 13 and above: The authorization state of Core Bluetooth manager.
  /// When running < iOS 13 or Android this is always allowed.
  static const bluetooth = p.Permission.bluetooth;

  /// Android: Allows an application a broad access to external storage in
  /// scoped storage.
  /// iOS: Nothing
  ///
  /// You should request the Manage External Storage permission only when
  /// your app cannot effectively make use of the more privacy-friendly APIs.
  /// For more information: https://developer.android.com/training/data-storage/manage-all-files
  ///
  /// When the privacy-friendly APIs (i.e. [Storage Access Framework](https://developer.android.com/guide/topics/providers/document-provider)
  /// or the [MediaStore](https://developer.android.com/training/data-storage/shared/media) APIs) is all your app needs the
  /// [PermissionGroup.storage] are the only permissions you need to request.
  ///
  /// If the usage of the Manage External Storage permission is needed,
  /// you have to fill out the Permission Declaration Form upon submitting
  /// your app to the Google Play Store. More details can be found here:
  /// https://support.google.com/googleplay/android-developer/answer/9214102#zippy=
  static const manageExternalStorage = p.Permission.manageExternalStorage;

  ///Android: Allows an app to create windows shown on top of all other apps
  ///iOS: Nothing
  static const systemAlertWindow = p.Permission.systemAlertWindow;

  ///Android: Allows an app to request installing packages.
  ///iOS: Nothing
  static const requestInstallPackages = p.Permission.requestInstallPackages;

  ///Android: Nothing
  ///iOS: Allows user to accept that your app collects data about end users and
  ///shares it with other companies for purposes of tracking across apps and
  ///websites.
  static const appTrackingTransparency = p.Permission.appTrackingTransparency;
}
